/*
 *
 * Copyright 2017 by Kappich Systemberatung Aachen
 * 
 * This file is part of de.bsvrz.ars.ars.
 * 
 * de.bsvrz.ars.ars is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * de.bsvrz.ars.ars is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with de.bsvrz.ars.ars.  If not, see <http://www.gnu.org/licenses/>.

 * Contact Information:
 * Kappich Systemberatung
 * Martin-Luther-Straße 14
 * 52062 Aachen, Germany
 * phone: +49 241 4090 436 
 * mail: <info@kappich.de>
 */

package de.bsvrz.ars.ars.mgmt.datatree.lock;

import de.bsvrz.sys.funclib.kappich.annotations.NotNull;
import de.bsvrz.sys.funclib.kappich.annotations.Nullable;

import java.util.concurrent.TimeUnit;
import java.util.concurrent.locks.ReentrantReadWriteLock;

/**
 * TBD Dokumentation
 *
 * @author Kappich Systemberatung
 */
@SuppressWarnings("LockAcquiredButNotSafelyReleased")
public final class DinProtector {

	private final MyReentrantReadWriteLock _delegate = new MyReentrantReadWriteLock();

	@Nullable
	public DinLock acquireReadLock(long timeout, final TimeUnit timeUnit) throws InterruptedException {
		if(_delegate.readLock().tryLock(timeout, timeUnit)) {
			return new DinLock(_delegate.readLock());
		}
		return null;
	}

	@Nullable
	public DinLock acquireWriteLock(long timeout, final TimeUnit timeUnit) throws InterruptedException {
		if(_delegate.writeLock().tryLock(timeout, timeUnit)) {
			return new DinLock(_delegate.writeLock());
		}
		return null;
	}

	@NotNull
	public DinLock acquireReadLock() throws InterruptedException {
		_delegate.readLock().lock();
		return new DinLock(_delegate.readLock());
	}

	@NotNull
	public DinLock acquireWriteLock() throws InterruptedException {
		_delegate.writeLock().lock();
		return new DinLock(_delegate.writeLock());
	}

	public boolean mayRead() {
		return _delegate.getReadHoldCount() != 0;
	}

	public boolean mayWrite() {
		return _delegate.isWriteLocked();
	}

	public Thread getWriteThread() {
		return _delegate.getOwner();
	}

	private static class MyReentrantReadWriteLock extends ReentrantReadWriteLock {
		@Override
		public Thread getOwner() {
			return super.getOwner();
		}
	}
}
